@@ -28,6 +28,7 @@ CONFIGFILES = [
 #: default values for configuration
 DEFAULTS = {
     'url': 'https://gateway.smstrade.de/',
+    'balanceurl': 'https://gateway.smstrade.de/credits/',
     'route': 'basic',
     'debug': 'True',  # debug is set to True to avoid costs
     'enable-cost': 'False',
@@ -109,9 +110,62 @@ class SMSTradeError(Exception):
         return self.message.encode(getpreferredencoding())
 
 
-class SMSTradeAPI(object):
+class SMSTradeAPIBase(object):
     """
-    Abstraction of the `smstrade.eu <http://smstrade.eu>`_ http(s) API.
+    Base class for smstrade.eu API clients.
+
+    """
+
+    def __init__(self, config=None, section='smstrade'):
+        """
+        Initialize a new SMSTradeAPIBase instance.
+
+        :param ConfigParser config:
+            use the specified configuration instead of the default
+            configuration
+
+        :param str section:
+            use the specified section in the configuration
+
+        """
+        if config:
+            self.config = config
+        else:
+            self.config = SafeConfigParser(defaults=DEFAULTS)
+            conffiles = self.config.read(CONFIGFILES)
+            if not conffiles:
+                log.warning((
+                    u'none of the configuration files (%s) found, trying'
+                    u' to continue with default values and command line'
+                    u' arguments'), ", ".join(CONFIGFILES))
+                self.config.add_section(section)
+        self.key = None
+        if self.config.has_section(section):
+            try:
+                self.key = self.config.get(section, 'key')
+            except NoOptionError as e:
+                log.warning(u"configuration is incomplete: %s", e)
+        else:
+            log.warning(u"configuration section %s does not exist.", section)
+
+    def prepare_args(self, kwargs):
+        """
+        Take a dictionary with keyword arguments, usually parsed from the
+        command line and set existing attributes of this instance.
+
+        :param dict kwargs:
+            dictionary of keyword arguments
+
+        """
+        for key, value in kwargs.iteritems():
+            if value is not None:
+                setattr(self, key, value)
+
+
+class SMSTradeAPI(SMSTradeAPIBase):
+    """
+    Abstraction of the `smstrade.eu <http://smstrade.eu>`_ http(s) mail sending
+    API.
 
     """
 
@@ -127,41 +181,30 @@ class SMSTradeAPI(object):
             use the specified section in the configuration
 
         """
-        if config:
-            self.config = config
-        else:
-            config = SafeConfigParser(defaults=DEFAULTS)
-            conffiles = config.read(CONFIGFILES)
-            if not conffiles:
-                log.warning((
-                    u'none of the configuration files (%s) found, trying'
-                    u' to continue with default values and command line'
-                    u' arguments'), ", ".join(CONFIGFILES))
-                config.add_section(section)
+        super(SMSTradeAPI, self).__init__(config, section)
 
         self.response = None
-        self.key = None
         self.sender = None
         self.reference = None
         self.senddate = None
         self.messagetype = None
         self.udh = None
-        if config.has_section(section):
+        if self.config.has_section(section):
             try:
-                self.url = config.get(section, 'url')
-                self.route = config.get(section, 'route')
-                self.debug = config.getboolean(section, 'debug')
-                self.cost = config.getboolean(section, 'enable-cost')
-                self.message_id = config.getboolean(
+                self.url = self.config.get(section, 'url')
+                self.route = self.config.get(section, 'route')
+                self.debug = self.config.getboolean(section, 'debug')
+                self.cost = self.config.getboolean(section, 'enable-cost')
+                self.message_id = self.config.getboolean(
                     section, 'enable-message-id')
-                self.count = config.getboolean(section, 'enable-count')
-                self.reports = config.getboolean(
+                self.count = self.config.getboolean(section, 'enable-count')
+                self.reports = self.config.getboolean(
                     section, 'enable-delivery-reports')
-                self.response = config.getboolean(section, 'enable-response')
-                self.concat = config.getboolean(section, 'enable-concat')
-                self.charset = config.get(section, 'charset')
-                self.key = config.get(section, 'key')
-                self.sender = config.get(section, 'from')
+                self.response = self.config.getboolean(
+                    section, 'enable-response')
+                self.concat = self.config.getboolean(section, 'enable-concat')
+                self.charset = self.config.get(section, 'charset')
+                self.sender = self.config.get(section, 'from')
             except NoOptionError as e:
                 log.warning(u"configuration is incomplete: %s", e)
         else:
@@ -442,9 +485,7 @@ class SMSTradeAPI(object):
             keyword arguments that override values in the configuration files
 
         """
-        for key, value in kwargs.iteritems():
-            if value is not None:
-                setattr(self, key, value)
+        self.prepare_args(kwargs)
 
         # convert to unicode
         if not self.messagetype == 'binary':
@@ -457,7 +498,61 @@ class SMSTradeAPI(object):
         return retval
 
 
+class SMSTradeBalanceAPI(SMSTradeAPIBase):
+    """
+    Abstraction of the `smstrade.eu <http://smstrade.eu>`_ http(s) account
+    balance API.
+
+    """
+
+    def __init__(self, config=None, section='smstrade'):
+        """
+        Initialize a new SMSTradeBalanceAPI instance.
+
+        :param ConfigParser config:
+            use the specified configuration instead of the default
+            configuration
+
+        :param str section:
+            use the specified section in the configuration
+
+        """
+        super(SMSTradeBalanceAPI, self).__init__(config, section)
+
+        if self.config.has_section(section):
+            self.url = self.config.get(section, 'balanceurl')
+        else:
+            log.warning(u"configuration section %s does not exist.", section)
+        log.debug(self.__dict__)
+
+    def get_balance(self, **kwargs):
+        """
+        Get the account balance from smstrade.eu and return it as a
+        :pyclass:`decimal.Decimal` instance.
+
+        :param dict kwargs:
+            keyword arguments that override values in the configuration files
+
+        """
+        self.prepare_args(kwargs)
+        if self.key is None:
+            raise SMSTradeError(
+                u'you need to define an API key either in a configuration'
+                u' file or on the command line')
+        response = requests.get(self.url, params={'key': self.key})
+        response.raise_for_status()
+        return decimal.Decimal(response.text)
+
+
 def hexstr(value):
+    """
+    Check whether the given hexadecimal value can successfully be decoded to
+    bytes.
+
+    :param str value:
+        string of hexadecimal representations of bytes
+
+    """
     try:
         value.lower().decode('hex')
     except TypeError as te:
@@ -466,6 +561,15 @@ def hexstr(value):
 
 
 def secondssinceepoch(seconds):
+    """
+    Check whether the given value is a valid timestamp after the current
+    timestamp.
+
+    :param str seconds:
+        string value that represents an int specifying a timestamp relative to
+        the UNIX epoch
+
+    """
     value = int(seconds)
     if datetime.fromtimestamp(value) < datetime.now():
         raise argparse.ArgumentTypeError(
@@ -473,7 +577,13 @@ def secondssinceepoch(seconds):
     return value
 
 
-def main(cmdline=None):
+def send_sms(cmdline=None):
+    """
+    Send SMS from the command line.
+
+    :param list cmdline:
+        list of command line arguments
+    """
     parser = argparse.ArgumentParser(
         description="Send one or more SMS via the smstrade.eu API")
     parser.add_argument(
@@ -544,6 +654,7 @@ def main(cmdline=None):
         'smstext',
         type=str,
         help="the text content of the SMS message to send")
+
     if cmdline is not None:
         args = parser.parse_args(cmdline)
     else:
@@ -566,3 +677,45 @@ def main(cmdline=None):
         log.info("Return value:\n%s", pformat(retval))
     except SMSTradeError as e:
         log.error(e)
+
+
+def account_balance(cmdline=None):
+    """
+    Get the smstrade.eu account balance from the command line.
+
+    :param list cmdline:
+        list of command line arguments
+    """
+    parser = argparse.ArgumentParser(
+        description="Send one or more SMS via the smstrade.eu API")
+    parser.add_argument(
+        '-c', '--config', type=file,
+        help=(
+            "alternative configuration instead of the default configuration"
+            "files:\n {0}").format(", ".join(CONFIGFILES)))
+    parser.add_argument(
+        '-s', '--section', type=str, default='smstrade',
+        help='use the specified section in the configuration file')
+    parser.add_argument(
+        '-k', '--key', type=str,
+        help="personal identification code")
+
+    if cmdline is not None:
+        args = parser.parse_args(cmdline)
+    else:
+        args = parser.parse_args()
+
+    config = None
+    if args.config:
+        config = SafeConfigParser(defaults=DEFAULTS)
+        config.readfp(args.config)
+
+    smstradebalance = SMSTradeBalanceAPI(
+        config=config, section=args.section)
+    del args.config
+    del args.section
+    try:
+        balance = smstradebalance.get_balance(**args.__dict__)
+        log.info(u"Your smstrade balance : %.03f €", balance)
+    except SMSTradeError as e:
+        log.error(e)
